#!/usr/bin/python

import os
import subprocess
from subprocess import PIPE
import re
import threading 
import time
import sys


ptpcam_exec=os.path.expandvars("$HOME")+"/Dokumente/robo-chdk/chdk-de/tools/ptpcam/ptpcam"



class Ptpcam:
    def __init__(self, bus, dev): 
        self.bus = bus
        self.dev = dev
        self.ptpcam_output=[]
        
    def call(self, cmd):
        self.ptpcam_output=[]
        for l in cmd:
            args = [ptpcam_exec, "--bus=%s"%self.bus, "--dev=%s"%self.dev, l.strip()]
            self.ptpcam_output.append(subprocess.check_output(args))
        return self.ptpcam_output
            
    def __str__(self):
        args = [ptpcam_exec, "--bus=%s"%self.bus, "--dev=%s"%self.dev, "--info"]
        return subprocess.check_output(args)


class PtpcamPipe:
    def __init__(self, bus, dev): 
        self.bus = bus
        self.dev = dev
        args = [ptpcam_exec, "--bus=%s"%self.bus, "--dev=%s"%self.dev, "--chdk"]
        self.subp = p=subprocess.Popen(args, shell=False, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        self.name = ""

        #self.preader = PipeReader(self.subp.stdout)
        #self.preader.start()
    
    def setName(self, name):
        self.name = name
        
    def call(self, cmd):
        self.subp.stdin.write(cmd)
        self.output = self.subp.stdout.readline()
        
    def write(self, cmd):
        self.subp.stdin.write(cmd)
        #self.output = []

    def clear_stdout(self):
        #self.subp.stdin.write("lua write_usb_msg(\"supper\")\n")
        self.subp.stdin.write("script-status\n")
        line=""
        while not re.search(b'script-status:0x[0-9]+ run=(no|yes) msg=(no|yes)', line):
            line = self.subp.stdout.readline()
            #print(line)
    def isscript_running(self):
        #self.subp.stdin.write("lua write_usb_msg(\"supper\")\n")
        self.subp.stdin.write("script-status\n")
        line=""
        while not re.search(b'script-status:0x[0-9]+ run=(no|yes) msg=(no|yes)', line):
            line = self.subp.stdout.readline()
        result = re.findall(b'script-status:0x[0-9]+ run=(no|yes) msg=(no|yes)', line) 
        if len(result) != 2:
            return False
        return result[0] == "yes"

    
    def async_readline(self):
        return PipeReader(self.subp.stdout)
        #while True:
            #time.sleep(0.1)
            #if len(self.output) == len(preader.output):
                #break
            #self.output = preader.output
        #preader.stop()

class PtpcamPipeSim:
    def __init__(self, bus, dev): 
        self.bus = bus
        self.dev = dev

        
    def call(self, cmd):
        sys.stdout.write("PtpcamPipeSim::call(cmd:%s)\n" % cmd)
      
    def write(self, cmd):
        sys.stdout.write("PtpcamPipeSim::write(cmd:%s)\n" % cmd)

    def async_readline(self):
        return PipeReaderSim(None)
        

                 

class PipeReader(threading.Thread):
    def __init__(self, pipe):
        threading.Thread.__init__(self) 
        self.pipe = pipe
        self.output = ""
    
    def run(self):
        self.output = self.pipe.readline()

class PipeReaderSim(threading.Thread):
    def __init__(self, pipe):
        threading.Thread.__init__(self) 
        self.pipe = pipe
        self.output = ""
    
    def run(self):
        self.output = "PipeReaderSim::run"

def getptpcams():
    ptpcam_output = subprocess.check_output([ptpcam_exec, "-l"])
    ptpdevmatch = re.findall(b'^([0-9]{3})/([0-9]{3})\s*0x[^/]*/0x\S*', ptpcam_output, re.MULTILINE)
    noptpdev = re.search(b'Found no PTP devices', ptpcam_output)
    if noptpdev:
        return []
    ptpcam_list=[]
    for (bus,dev) in ptpdevmatch:
        #ptpcam_output = subprocess.check_output([ptpcam_exec, "--bus="+str(bus), "--dev="+str(dev), "-i"])
        ptpcam_list.append(Ptpcam(bus.decode(), dev.decode()))
    return ptpcam_list
    
def getptpcamsPipe():
    ptpcam_output = subprocess.check_output([ptpcam_exec, "-l"])
    ptpdevmatch = re.findall(b'^([0-9]{3})/([0-9]{3})\s*0x[^/]*/0x\S*', ptpcam_output, re.MULTILINE)
    noptpdev = re.search(b'Found no PTP devices', ptpcam_output)
    if noptpdev:
        return []
    ptpcam_list=[]
    for (bus,dev) in ptpdevmatch:
        #ptpcam_output = subprocess.check_output([ptpcam_exec, "--bus="+str(bus), "--dev="+str(dev), "-i"])
        ptpcam_list.append(PtpcamPipe(bus.decode(), dev.decode()))
    return ptpcam_list